'use strict'

exports.__esModule = true
exports.useAsync = useAsync
exports.useAsyncEffect = useAsyncEffect

var _react = _interopRequireDefault(require('react'))

var _useMemoOne = require('use-memo-one')

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj}
}

const {useReducer, useEffect} = _react.default

function _ref(prev, action) {
  return {
    status: action.status,
    value: action.status === 'success' ? action.value : void 0,
    error: action.status === 'error' ? action.error : void 0,
    id: action.status === 'cancelled' ? ++prev.id : prev.id,
  }
}

function useAsync(asyncCallback, dependencies = []) {
  const [state, dispatch] = useReducer(_ref, {
    status: 'idle',
    value: void 0,
    error: void 0,
    id: 0,
  }) // Creates a stable callback that manages our loading/success/error status updates
  // as the callback is invoked.

  const callback = (0, _useMemoOne.useCallbackOne)(
    Object.assign(
      async (...args) => {
        dispatch({
          status: 'loading',
        })

        try {
          const value = await asyncCallback(...args)
          if (callback.cancelled) return
          dispatch({
            status: 'success',
            value,
          })
        } catch (error) {
          if (callback.cancelled) return
          dispatch({
            status: 'error',
            error,
          })
        }
      },
      {
        cancelled: false,
      }
    ), // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies.concat(state.id)
  ) // Cancel any pending async stuff on unmount

  function _ref2() {
    callback.cancelled = true
  }

  useEffect(() => _ref2, [callback])

  function _ref3() {
    return dispatch({
      status: 'cancelled',
    })
  }

  return [
    (0, _useMemoOne.useMemoOne)(
      () => ({
        status: state.status,
        value: state.value,
        error: state.error,
        cancel: _ref3,
      }),
      [state]
    ),
    callback,
  ]
}

function useAsyncEffect(asyncCallback, dependencies) {
  const [state, callback] = useAsync(
    asyncCallback,
    dependencies || [Math.random()]
  ) // Runs the callback each time the dependencies change and cancels any
  // pending callbacks that were running previously

  useEffect(
    () => {
      callback()
    }, // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  )
  return state
}
